// 初期設定ファイルの操作
// 現在はポップアップメニュー、つまり外部ビューアーと地図表示に付いてのみ対応
// ファイル名：.srchphoto
// 内容：
//	[popupmenu]
//	title0=文字列				← ビューワー指定
//	action0=command				← コマンド文字列
//	arg1						← 引数1
//	arg2
//	title1=文字列				← 地図表示 - 国土地理印
//	action1=command
//	arg1
//	arg2
//	title2=文字列				← 地図表示 - Google Map
//	action2=command
//	arg1
//	arg2
// '='から行末までは有効
// $F:	JPEGファイルのフルパス
// $X:	経度
// $Y:	緯度
import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;

public class Config {
	BufferedReader Reader;
	// 初期設定ファイル名
	private static final String ConfigNameLinux = ".srchphoto";
	private static final String ConfigNameWin = "srchphoto.ini";
	private static final String ConfigNameMac = "";
	// 初期設定用見出し
	private static final String BlockpmenuStr		= "[popupmenu]";
	private static final String PmenuTitle0Str	= "title0=";
	private static final String PmenuAction0Str	= "action0=";
	private static final String PmenuTitle1Str	= "title1=";
	private static final String PmenuAction1Str	= "action1=";
	private static final String PmenuTitle2Str	= "title2=";
	private static final String PmenuAction2Str	= "action2=";
	// 既定のメニュー見出し
	private static final String DefaultTitle0 = "外部ビューアで表示";
	private static final String DefaultTitle1 = "撮影場所の地図(電子国土)";
	private static final String DefaultTitle2 = "撮影場所の地図(Google Map)";
	private static final String[] DefaultAaction0Linux = { "eog", "$F" };
	private static final String[] DefaultAaction1Linux = { "firefox", "http://portal.cyberjapan.jp/site/mapuse/index.html?longitude=$X&latitude=$Y" };
	private static final String[] DefaultAaction2Linux = { "firefox", " http://maps.google.co.jp/maps?ll=$Y,$X&spn=0.01,0.01&q=$Y,$X" };
	private static final String[] DefaultAaction0Win = { "C:\\Windows\\system32\\mspaint.exe", "$F" };
	private static final String[] DefaultAaction1Win = { "C:\\Program Files\\Internet Explorer\\iexplore.exe", "http://portal.cyberjapan.jp/site/mapuse/index.html?longitude=$X&latitude=$Y" };
	private static final String[] DefaultAaction2Win = { "C:\\Program Files\\Internet Explorer\\iexplore.exe", "http://maps.google.co.jp/maps?ll=$Y,$X&spn=0.01,0.01&q=$Y,$X" };
	private static final String[] DefaultAaction0Mac = { "", "$F" };
	private static final String[] DefaultAaction1Mac = { "", "http://portal.cyberjapan.jp/site/mapuse/index.html?longitude=$X&latitude=$Y" };
	private static final String[] DefaultAaction2Mac = { "", " http://maps.google.co.jp/maps?ll=$Y,$X&spn=0.01,0.01&q=$Y,$X" };
	// 実行コマンド中の置き換え用正規表現
	static final String PathReplace = "$F";
	static final String LonRepalce = "$X";
	static final String LatRepalce = "$Y";
	
	private String pmenuTitle0;
	private List<String> pmenuaction0 = new ArrayList<String>();
	private String pmenuTitle1;
	private List<String> pmenuaction1 = new ArrayList<String>();
	private String pmenuTitle2;
	private List<String> pmenuaction2 = new ArrayList<String>();
	
	// 初期設定された項目の入手
	String getpmenuTitle0() {
		return pmenuTitle0;
	}
	List<String> getpmenuaction0() {
		return pmenuaction0;
	}
	String getpmenuTitle1() {
		return pmenuTitle1;
	}
	List<String> getpmenuaction1() {
		return pmenuaction1;
	}
	String getpmenuTitle2() {
		return pmenuTitle2;
	}
	List<String> getpmenuaction2() {
		return pmenuaction2;
	}
	
	// 設定ファイルが見当たらない場合など、デフォルト値を設定
	void setDefaultConfig() {
		int	i;
		pmenuTitle0 = DefaultTitle0;
		pmenuTitle1 = DefaultTitle1;
		pmenuTitle2 = DefaultTitle2;
		// OSの種類によって、コマンドを変える
		String OSName = System.getProperty("os.name");
		if (OSName == null)
			OSName = "Windows";
		if (OSName.startsWith("Linux")) {
			for (i = 0; i < DefaultAaction0Linux.length; i++)
				pmenuaction0.add(DefaultAaction0Linux[i]);
			for (i = 0; i < DefaultAaction1Linux.length; i++)
				pmenuaction1.add(DefaultAaction1Linux[i]);
			for (i = 0; i < DefaultAaction2Linux.length; i++)
				pmenuaction2.add(DefaultAaction2Linux[i]);
		}
		else if (OSName.startsWith("Windows")) {
			for (i = 0; i < DefaultAaction0Win.length; i++)
				pmenuaction0.add(DefaultAaction0Win[i]);
			for (i = 0; i < DefaultAaction1Win.length; i++)
				pmenuaction1.add(DefaultAaction1Win[i]);
			for (i = 0; i < DefaultAaction2Win.length; i++)
				pmenuaction2.add(DefaultAaction2Win[i]);
		}
		else if (OSName.startsWith("Mac")) {
			for (i = 0; i < DefaultAaction0Mac.length; i++)
				pmenuaction0.add(DefaultAaction0Mac[i]);
			for (i = 0; i < DefaultAaction1Mac.length; i++)
				pmenuaction1.add(DefaultAaction1Mac[i]);
			for (i = 0; i < DefaultAaction2Mac.length; i++)
				pmenuaction2.add(DefaultAaction2Mac[i]);
		}
		else {
			// I don't know what to do.
		}
	}
	
	public Config() {
		setDefaultConfig();
		try {
			String line;
			String home = System.getProperty("user.home");
			String separator = System.getProperty("file.separator");
			
			String OSName = System.getProperty("os.name");
			if (OSName.startsWith("Linux"))
				Reader = new BufferedReader(new FileReader(new File(home + separator + ConfigNameLinux)));
			else if (OSName.startsWith("Windows"))
				Reader = new BufferedReader(new FileReader(new File(home + separator + ConfigNameWin)));
			else if (OSName.startsWith("Mac"))
				Reader = new BufferedReader(new FileReader(new File(home + separator + ConfigNameMac)));
			
			while ((line = Reader.readLine()) != null) {
				if (line.equals(BlockpmenuStr) == true)
					break;				// "[popupmenu]" が出現するまで、読み飛ばし
			}
			if ((line = Reader.readLine()) == null)
				return;
			if (line.startsWith(PmenuTitle0Str) == true) {		// メニュータイトル?
				// 外部ビューアのメニュータイトルを取得
				pmenuTitle0 = line.substring(PmenuTitle0Str.length());
				if ((line = Reader.readLine()) == null)
					return;
				if (line.startsWith(PmenuAction0Str) == true) {
					// コマンド名
					pmenuaction0.clear();
					pmenuaction0.add(line.substring(PmenuAction0Str.length()));
					// 引数を読み取る
					while (true) {
						if ((line = Reader.readLine()) == null)
							return;
						if (line.startsWith(PmenuTitle1Str) == true)
							break;		// 次のメニュー項目
						pmenuaction0.add(line);
					}
				}
			}
			if (line.startsWith(PmenuTitle1Str) == true) {
				pmenuTitle1 = line.substring(PmenuTitle1Str.length());
				if ((line = Reader.readLine()) == null)
					return;
				if (line.startsWith(PmenuAction1Str) == true) {
					pmenuaction1.clear();
					pmenuaction1.add(line.substring(PmenuAction1Str.length()));
					while (true) {
						if ((line = Reader.readLine()) == null)
							return;
						if (line.startsWith(PmenuTitle2Str) == true)
							break;		// 次のメニュー項目
						pmenuaction1.add(line);
					}
				}
			}
			if (line.startsWith(PmenuTitle2Str) == true) {
				pmenuTitle2 = line.substring(PmenuTitle2Str.length());
				if ((line = Reader.readLine()) == null)
					return;
				if (line.startsWith(PmenuAction2Str) == true) {
					pmenuaction2.clear();
					pmenuaction2.add(line.substring(PmenuAction2Str.length()));
					while (true) {
						if ((line = Reader.readLine()) == null)
							return;
						pmenuaction2.add(line);
					}
				}
			}
		}
		catch (IOException e) {
		}
		finally {
			if (Reader != null) {
				try {
					Reader.close();
				}
				catch (IOException e) {
				}
			}
		}
	}
}
