import java.io.File;
import java.lang.Math;
import java.util.List;

import javax.swing.JOptionPane;


class SearchCondition {
	String		SearchPath;
	boolean	DateValid;
	int			YearS;
	int			YearE;
	boolean	EveryY;
	int			MonS;
	int			MonE;
	boolean	EveryM;
	int			DayS;
	int			DayE;
	boolean	CoodValid;
	char		LatRef;
	double		LatDMI;
	int			LatDeg;
	int			LatMin;
	double		LatSec;
	char		LongRef;
	double		LongDMI;
	int			LongDeg;
	int			LongMin;
	double		LongSec;
	int			Scope;
	double		LimitN;			// 南半球は負数
	double		LimitS;			//		〃
	double		LimitE;			// 西半球は負数
	double		LimitW;			//		〃
	List<String>	CaptionList;
	List<String>	TagList;

	String getPath() { return SearchPath; }		// 検索対象フォルダを返す
	
	// 設定ダイアログから検索条件を取り出す
	void GetCondition() {
		SearchPath = AppWin.getSearchFolder();
		if (SearchPath.isEmpty() != true && SearchPath.endsWith(File.separator) != true)
			SearchPath += File.separator;
		if (AppWin.isDateValid() == true) {
			DateValid = true;
			YearS = AppWin.getYearS();
			YearE = AppWin.getYearE();
			EveryY = AppWin.getEveryY();
			MonS = AppWin.getMonS();
			MonE = AppWin.getMonE();
			EveryM = AppWin.getEveryM();
			DayS = AppWin.getDayS();			// 月末は 31
			DayE = AppWin.getDayE();			// 月末は 31
		}
		else
			DateValid = false;
		if (AppWin.isCoodValid() == true) {
			CoodValid = true;
			LatRef = AppWin.getLatRef();
			LatDMI = AppWin.getLatDMI();
			LatDeg = AppWin.getLatDeg();
			LatMin = AppWin.getLatMin();
			LatSec = AppWin.getLatSec();
			LongRef = AppWin.getLongRef();
			LongDMI = AppWin.getLongDMI();
			LongDeg = AppWin.getLongDeg();
			LongMin = AppWin.getLongMin();
			LongSec = AppWin.getLongSec();
			Scope = AppWin.getScope();
		}
		else
			CoodValid = false;
		
		CalLimit();							// 中心座標と範囲から東西南北の境界値を求める
		
		if (AppWin.chckbxNewCheckBox_2.isSelected() == true)
			CaptionList = AppWin.CaptionList.GetList();
		if (AppWin.chckbxNewCheckBox_3.isSelected() == true)
			TagList = AppWin.TagList.GetList();
	}
	
	// 中心座標と範囲から東西南北の境界値を求める
	private void CalLimit() {
		final double RadiusP = 6356752;		// 地球の極半径   単位 m
		final double RadiusE = 6378137;		//	 〃  赤道半径 単位 m
		double Angle;
		double R;
		// 南北方向を計算
		Angle = (Scope * 180) / (Math.PI * RadiusP);
		LimitN = LimitS = (LatRef == 'N') ? LatDMI : -LatDMI;
		LimitN += (Angle / 2);
		LimitS -= (Angle / 2);
		// 東西方向を計算
		R = RadiusE * Math.cos(Math.PI * (LatDMI / 180));	// 緯度に対応した半径を求める
		Angle = (Scope * 180) / (Math.PI * R);
		LimitE = LimitW = (LongRef == 'E') ? LongDMI : -LongDMI;
		LimitE += (Angle / 2);
		LimitW -= (Angle / 2);
	}
	
	// ダイアログに設定された値をチェックする
	boolean isOK() {
		String BadDate = "日付の指定が正しくありません";
		String BadLat = "緯度の指定が正しくありません";
		String BadLong = "経度の指定が正しくありません";
		String Warning = "警告";
		
		if (SearchPath.isEmpty()) {
			// フォルダが指定されていない
			JOptionPane.showMessageDialog(null, "検索対象フォルダを指定してください", Warning, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		File TopFolder = new File(SearchPath);
		if (TopFolder.isDirectory() != true) {
			// 指定されたパスが存在しないか、フォルダでない
			JOptionPane.showMessageDialog(null, "指定されたフォルダが存在しません", Warning, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		if ((YearS != 0) && (YearE != 0)) {
			// 開始、終了どちらも指定された
			if ((YearS < 1900) || (YearS > 2100) || (YearE < 1900) || (YearE > 2100)) {
				// 年が 1900年より前、または2100年より後
				JOptionPane.showMessageDialog(null, BadDate, Warning, JOptionPane.ERROR_MESSAGE);
				return false;
			}
			if (YearS > YearE) {
				// 開始年が終了年より大きい
				JOptionPane.showMessageDialog(null, BadDate, Warning, JOptionPane.ERROR_MESSAGE);
				return false;
			}
		}
		if (((YearS == 0) && (YearE != 0)) || ((MonS == 0) && (MonE != 0)) || ((DayS == 0) && (DayE != 0))) {
			// 開始[年月日]が指定されていないのに、終了[年月日]が指定されている
			JOptionPane.showMessageDialog(null, BadDate, Warning, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		if (((YearS == 0) && (MonS == 0) && (DayS == 0)) && ((YearE != 0) || (MonE != 0) || (DayE != 0))) {
			// 日付はすべて空欄か、そうでなければ開始[年月日]は指定されていなければならない
			JOptionPane.showMessageDialog(null, BadDate, Warning, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		if ((YearS != 0) && (YearE != 0) && (MonE == 0) && (EveryY == false)) {
			// 複数年が指定されているが、終了月が指定されておらず、尚且つ「各年」がチェックされていない
			JOptionPane.showMessageDialog(null, BadDate, Warning, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		if ((MonS != 0) && (MonE != 0) && (DayE == 0) && (EveryM == false)) {
			// 複数月が指定されているが、終了日が指定されておらず、尚且つ「各月」がチェックされていない
			JOptionPane.showMessageDialog(null, BadDate, Warning, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		if (((MonS < 0) || (12 < MonS)) || ((MonE < 0) || (12 < MonE))) {
			// 月がⅠ～12以外
			JOptionPane.showMessageDialog(null, BadDate, Warning, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		if (((DayS < 0) || (31 < DayS)) || ((DayE < 0) || (31 < DayE))) {
			// 日が1～31以外
			JOptionPane.showMessageDialog(null, BadDate, Warning, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		if ((LatDeg < 0) || (90 < LatDeg) || (LatMin < 0) || (59 < LatMin) || (LatSec < 0) || (60 <= LatSec)) {
			// 緯度の度分秒が範囲外
			JOptionPane.showMessageDialog(null, BadLat, Warning, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		if ((LatDMI < 0) || (90 < LatDMI)) {
			// 実数形式の緯度が範囲外
			JOptionPane.showMessageDialog(null, BadLat, Warning, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		if ((LongDeg < 0) || (180 < LongDeg) || (LongMin < 0) || (59 < LongMin) || (LongSec < 0) || (60 <= LongSec)) {
			// 経度の度分秒が範囲外
			JOptionPane.showMessageDialog(null, BadLong, Warning, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		if ((LongDMI < 0) || (180 < LongDMI)) {
			// 実数形式の経度が範囲外
			JOptionPane.showMessageDialog(null, BadLong, Warning, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		return true;
	}
}
