import java.io.FileFilter;
import java.io.File;
import java.util.List;

import java.awt.Container;
import java.awt.Component;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;


class JpegFilter implements FileFilter {			// Jpegファイルフィルタ
	public boolean accept(File FilePath) {
		String LowCasePath = FilePath.getPath().toLowerCase();
		if (LowCasePath.lastIndexOf(".jpeg") == LowCasePath.length() - 5)
			return true;
		if (LowCasePath.lastIndexOf(".jpg")  == LowCasePath.length() - 4)
			return true;
		if (LowCasePath.lastIndexOf(".jpe")  == LowCasePath.length() - 4)
			return true;
		return false;
	}
}

class FolderFilter implements FileFilter {		// フォルダを選択
	public boolean accept(File FilePath) {
		return FilePath.isDirectory();
	}
}

// 検索スレッド
class SrchThread extends Thread {
	public void run() {
		SearchCondition Condition = new SearchCondition();		
		Condition.GetCondition();							// ダイアログから設定された条件を取り出す
		if (Condition.isOK() == false)						// 設定された条件をチェックする
			return;
		SrchPhoto Searcher = new SrchPhoto(Condition);		// 検索オブジェクトを作成
		AppWinHndl.setSearcher(Searcher);					// 検索オブジェクトを作成
		if (Searcher != null) {
			AppWin.setDisable();							// 条件設定のダイアログを操作不可にして
			Searcher.doSrch(Condition);						// 条件を引数にして検索を実施
			AppWin.setEnable();								// 操作可能に戻す
		}
	}
}

@SuppressWarnings("serial")
class PrgDialog extends JFrame {
	
	Container contentPane;
	JLabel lblNewLabel_1;
	JLabel lblNewLabel_2;
	static final String label1 = "検索フォルダー：";
	static final String label2 = "検出ファイル数：";
	enum runstate {
		STOP, RUN, ABORT
	};
	volatile runstate RunState = runstate.STOP;
	
	PrgDialog(String title) {
		super(title);
		setSize(400, 130);
		setResizable(false);
		contentPane = getContentPane();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
		
		JPanel panel1 = new JPanel();
		contentPane.add(panel1);
		panel1.setLayout(new BoxLayout(panel1, BoxLayout.Y_AXIS));
		
		lblNewLabel_1 = new JLabel(label1);
		panel1.add(lblNewLabel_1);
		
		lblNewLabel_2 = new JLabel(label2);
		panel1.add(lblNewLabel_2);
		JPanel panel2 = new JPanel();
		panel2.setAlignmentY(Component.BOTTOM_ALIGNMENT);
		contentPane.add(panel2);
		
		JButton btnNewButton = new JButton("中断");
		btnNewButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				confirm();
				if (RunState == runstate.ABORT)
					dispose();
			}
		});
		panel2.add(btnNewButton);
		
		setLocationRelativeTo(null);
	}
	
	@Override
	public void setVisible(boolean visible) {
		if (visible == true)
			RunState = runstate.RUN;
		else
			RunState = runstate.STOP;
		
		super.setVisible(visible);
	}
	
	// "中止"ボタンがクリックされたとき
	private void confirm() {
		int retval;
		
		RunState = runstate.STOP;
		retval = JOptionPane.showConfirmDialog(contentPane, "検索を中止しますか?", "検索中断", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
		if (retval == JOptionPane.YES_OPTION)
			RunState = runstate.ABORT;
		else
			RunState = runstate.RUN;
	}

	public Insets getInsets() {
		return new Insets(40, 10, 10, 10);
	}
	
	runstate getRunState() {
		return RunState;
	}
}
	
class SrchPhoto {
	JpegFilter		JpgFilter;
	FolderFilter	FldFilter;
	int				FileCount;
	int				TopPathLen;
	PrgDialog		progress;
	
	private static FileTbl FileTable = null;
	SrchPhoto(SearchCondition Condition) {
		if (FileTable != null) {
			FileTable.dispose();
		}
		FileTable = new FileTbl(Condition.getPath());
		FileCount = 0;
	}
	
	FileTbl getFileTbl() {
		return FileTable;
	}

	// ファイル検索のトップレベル
	boolean doSrch(SearchCondition Condition) {
		File Folder = new File(Condition.getPath());
		FldFilter = new FolderFilter();						// フォルダのみを抽出
		JpgFilter = new JpegFilter();						// "*.jpeg", "*.jpg", "*.jpe"のフィルタを作成する
		progress = new  PrgDialog("検索中…");				// Progress Bar を表示
		FileCount = 0;
		TopPathLen = Folder.getPath().length();
		progress.setVisible(true);
		FileTable.setEnable(false);
		SrchInFolder(Folder, Condition);					// 指定されたフォルダから再帰的に検索する
		FileTable.setEnable(true);
		progress.dispose();
		return true;
	}
	
	// '毎年', '毎月' 共に指定されていない場合、日付を判定する
	private boolean MatchCondDate(FileInfo info, SearchCondition Condition) {
		int		YearS = Condition.YearS;
		int		MonS = Condition.MonS;
		int		DayS = Condition.DayS;
		int		YearE = Condition.YearE;
		int		MonE = Condition.MonE;
		int		DayE = Condition.DayE;
		int		YMDS;
		int		YMDE;
		int		YMD;
		
		if (DayS == 31) {			// '末' 指定？
			// 31 を実際の数字に置き換え
			switch (MonS) {
			case 2:
				if ((YearS % 4) == 0 && (YearS % 100) != 0 && (YearS % 400) == 0)
					DayS = 29;
				else
					DayS = 28;
				break;
			case 4:
			case 6:
			case 9:
			case 11:
			default:
				DayS = 30;
				break;
			}
		}
		if (YearE == 0 || MonE == 0 || DayE == 0) {
			// 開始日 1日だけの指定
			if (info.getYear() == YearS && info.getMon() == MonS && info.getDay() == DayS)
				return true;
		}
		if (DayE == 31) {			// '末' 指定？
			// 31 を実際の数字に置き換え
			switch (MonE) {
			case 2:
				if ((YearE % 4) == 0 && (YearE % 100) != 0 && (YearE % 400) == 0)
					DayE = 29;
				else
					DayE = 28;
				break;
			case 4:
			case 6:
			case 9:
			case 11:
			default:
				DayE = 30;
				break;
			}
		}
		// 日付が指定期間内?
		YMDS = (YearS * 10000) + (MonS * 100) + DayS;
		YMDE = (YearE * 10000) + (MonE * 100) + DayE;
		YMD = (info.getYear() * 10000) + (info.getMon() * 100) + info.getDay();
		if (YMDS <= YMD && YMD <= YMDE)
				return true;
		
		return false;
	}
	
	// '毎年'が指定された場合
	// 指定された間、毎年のMonS, DayS から MonE, DyaE の間にあればtrueを返す
	private boolean MatchCondEveryY(FileInfo info, SearchCondition Condition) {
		int		YearS = Condition.YearS;
		int		MonS = Condition.MonS;
		int		DayS = Condition.DayS;
		int		YearE = Condition.YearE;
		int		MonE = Condition.MonE;
		int		DayE = Condition.DayE;
		int		YearCrossing = 0;
		int		YMDS;
		int		YMDE;
		int		YMD;

		if (MonS > MonE)
			YearCrossing = 1;			// 年を越している 12月31日～1月1日のような場合
		for (int Year = YearS; Year <= YearE; Year++) {
			// 日付が指定期間内?
			YMDS = (Year * 10000) + (MonS * 100) + DayS;
			YMDE = ((Year + YearCrossing) * 10000) + (MonE * 100) + DayE;
			YMD = (info.getYear() * 10000) + (info.getMon() * 100) + info.getDay();
			if (YMDS <= YMD && YMD <= YMDE)
					return true;
		}
		return false;
	}
	
	// '毎年' が指定されていなく、'毎月' が指定されている
	// 開始年月から終了年月の期間中、毎月指定された日
	private boolean MatchCondEveryM(FileInfo info, SearchCondition Condition) {
		int		YearS = Condition.YearS;
		int		MonS = Condition.MonS;
		int		DayS = Condition.DayS;
		int		YearE = Condition.YearE;
		int		MonE = Condition.MonE;
		int		DayE = Condition.DayE;
		int		YearCrossing = 0;
		int		MonCrossing = 0;
		int		YMDS;
		int		YMDE;
		int		YMD;

		YMD = (info.getYear() * 10000) + (info.getMon() * 100) + info.getDay();	// 撮影日時
		
		if (DayS > DayE) {
			// 期間終わりの日が翌月
			if (MonE != 12) {
				// 年を越すことはない
				MonCrossing = 100;
			}
			else {
				// 期間終わりの日が翌年
				YearCrossing = 10000;
			}
		}
		
		// 指定された年月の期間
		int YME = YearE * 10000 + MonE * 100;
		int	y = YearS;
		int	m = MonS;
		int ym = (y * 10000) + (m * 100);
		while (ym <= YME) {
			YMDS = ym + DayS;
			YMDE = ym + YearCrossing;
			if (YearCrossing != 0) {
				// 年を越す
				YMDE += (100 + DayE);		// 1月とする
			}
			else {
				YMDE += (MonCrossing + DayE);
			}
			if (YMDS <= YMD && YMD <= YMDE)
				return true;
			
			// 次の月
			if (m == 12) {
				y++;
				m = 1;
			}
			else
				m++;
		 	ym = (y * 10000) + (m * 100);
		}
		
		return false;
	}

	// '毎年', '毎月' の両方が指定されている
	// 開始年～終了年の毎年、指定された期間の毎月、指定された日
	private boolean MatchCondEveryYM(FileInfo info, SearchCondition Condition) {
		int		YearS = Condition.YearS;
		int		MonS = Condition.MonS;
		int		DayS = Condition.DayS;
		int		YearE = Condition.YearE;
		int		MonE = Condition.MonE;
		int		DayE = Condition.DayE;
		int		YMDS;
		int		YMDE;
		int		YMD;


		YMD = (info.getYear() * 10000) + (info.getMon() * 100) + info.getDay();	// 撮影日時

		for (int Year = YearS; Year <= YearE; Year++) {
			for (int Mon = MonS; Mon <= MonE; Mon++) {
				YMDS = Year * 10000 + (Mon * 100) + DayS;
				YMDE = Year * 10000;
				if (DayS > DayE) {
					// 期間終わりの日が翌月
					if (Mon != 12) {
						// 年を越すことはない
						YMDE += (Mon * 100) + 100 + DayE;
					}
					else {
						// 期間終わりの日が翌年
						YMDE += 10000 + 100 + DayE;		// 翌年の1月
					}
				}
				else {
					YMDE += (Mon * 100) + DayE;
				}
				if (YMDS <= YMD && YMD <= YMDE)
					return true;
			}
		}

		return false;
	}
	
	// XPKeywords と LastKeywordXMP のどちらと一致するか不明なので
	// 実際の検索は FileInfo にまかせる
	private boolean MatchCondTag(FileInfo info, List<String> SrchTags) {
		return info.MatchCondTag(SrchTags);
	}

	// JPEGファイルのタイトルが条件に一致するか調べる
	private boolean MatchCondCaption(FileInfo info, List<String> captionList) {
		String Title;
		
		Title = info.getTitle();
		if (Title == null)
			return false;					// 	タイトルが無い
		
		for (String s: captionList) {
			if (Title.indexOf(s) >= 0)
				return true;				// 指定された文字列を含むタイトルがあった
		}
		
		return false;
	}

	// JpegのTagが指定された条件と合致するかどうか判定する
	private boolean MatchCond(FileInfo info, SearchCondition Condition) {
		double infoLatDMS;
		double infoLongDMS;
		
		if (Condition.CoodValid == true) {
			// 検索条件に座標が指定されている
			if (info.getCoodValid() != 0) {
				infoLatDMS = (info.getLatRef() == 'N') ? info.getLatDMS() : -info.getLatDMS();
				infoLongDMS = (info.getLongRef() == 'E') ? info.getLongDMS() : -info.getLongDMS();
				if ((infoLatDMS < Condition.LimitS) || (Condition.LimitN < infoLatDMS))
					return false;
				if ((infoLongDMS < Condition.LimitW) || (Condition.LimitE < infoLongDMS))
					return false;
			}
			else						// Exif情報に座標が無い
				return false;
		}
		
		if (Condition.DateValid == true) {
			// 検索条件に日付が指定されている
			if (Condition.EveryY != true && Condition.EveryM != true) {
				// '毎年', '毎月' 共に指定されていない
				if (MatchCondDate(info, Condition) == false)
					return false;
			}
			if (Condition.EveryY == true && Condition.EveryM != true) {
				// '毎年'が指定されていて、'毎月'が指定されていない
				if (MatchCondEveryY(info, Condition) == false)
					return false;
			}
			if (Condition.EveryY != true && Condition.EveryM == true) {
				// '毎年' が指定されていなく、'毎月' が指定されている
				// 開始年月から終了年月の期間中、毎月指定された日
				if (MatchCondEveryM(info, Condition) == false)
					return false;;
			}
			if (Condition.EveryY == true && Condition.EveryM == true) {
				if (MatchCondEveryYM(info, Condition) == false)
					return false;
			}
		}
		
		if ((Condition.CaptionList != null) && (Condition.CaptionList.size() != 0)) {
			// キャプションの一致が条件
			if (MatchCondCaption(info, Condition.CaptionList) == false)
				return false;					// キャプションが一致しない
		}
		if ((Condition.TagList != null) && (Condition.TagList.size() != 0)) {
			// タグの一致が条件
			if (MatchCondTag(info, Condition.TagList) == false)
				return false;					// タグが一致しない
		}
		
		return true;
	}
	
	// 指定されたフォルダ内を検索する
	private void SrchInFolder(File Folder, SearchCondition Condition) {
		if (progress.RunState == PrgDialog.runstate.ABORT)
			return;
		
		// 進捗ダイアログの検索中フォルダを更新
		progress.lblNewLabel_1.setText(PrgDialog.label1 + Folder.getPath());
		// JPEGファイルを探す
		File[] FileList = Folder.listFiles(JpgFilter);
		int lenOfTopFolder = Condition.getPath().length();
		if (Folder.getPath().length() > TopPathLen)
			progress.lblNewLabel_1.setText(PrgDialog.label1 + Folder.getPath().substring(TopPathLen + 1));	// 検索中フォルダ "+1" は '/'
		else
			progress.lblNewLabel_1.setText(PrgDialog.label1 + Folder.getPath().substring(TopPathLen));
		for (File JpgFile: FileList) {			
			while (progress.RunState == PrgDialog.runstate.STOP)
				continue;
			if (progress.RunState == PrgDialog.runstate.ABORT)
				return;
			
			FileInfo info = new FileInfo(JpgFile);
			if (info.getFileInfo() != false) {
				// Geo Tagが見つかった
				if (MatchCond(info, Condition) == true) {
					// 条件が合致したら、
					// 検索対象フォルダ以下の文字列を取り出し、"+1"はセパレータ('/'や'\\'など)
					// テーブルに追加する。
					FileTable.AddRow(info, JpgFile.getPath().substring(lenOfTopFolder), info.getYMDHMSStr(), info.getLatDMSStr(), info.getLongDMSStr(), info.getTitle(), info.getTagString());
					FileCount++;
					progress.lblNewLabel_2.setText(PrgDialog.label2 + String.valueOf(FileCount));
				}
			}
		}
		// サブフォルダが存在すれば、再帰する
		File[] FolderList = Folder.listFiles(FldFilter);
		for (File subFolder: FolderList) {
			if (progress.RunState == PrgDialog.runstate.ABORT)
				return;
			SrchInFolder(subFolder, Condition);
		}
	}
}
